semantic_space <-
  function(
    formula = ~ Lexeme + Person + Number + Tense + Voice + Mood,  
    # the first class of the formula should be the base
    wordform = "Word",   # the name of the column in data that has the word forms 
    data=latin,
    with_semantic_vectors = TRUE,  # add for each lexome and inflectional feature a separate semantic vector
    with_mvrnorm = FALSE,          # if TRUE, construct semantic vectors with mvrnorm
    mean_for_content_lexomes=0,            # used both for mvrnorm and for semantic vectors
    mean_for_inflectional_features=0,      # used both for mvrnorm and for semantic vectors
    stdev_for_content_lexomes=1,           # stdev for variation in means for lexomes
    stdev_for_inflectional_features=1,     # stdev for variation in means for inflectional features
    covar_for_content_lexomes=40,          # used only when with_mvrnorm==TRUE
    covar_for_inflectional_features=10,    # used only when with_mvrnorm==TRUE
    stdev_for_lexomic_semantic_vectors = 4,        # stdev for the rnorm generating semantic vectors for lexomes
    stdev_for_inflectional_semantic_vectors = 4,   # stdev for the rnorm generating semantic vectors for infl.
    nrows=672,         # number of wordforms, equal to # rows of C
    ncols=256          # required number of features, equal to # columns of C
  ) {

    if (!with_semantic_vectors & !with_mvrnorm) {
      stop("illegal option combination\n")
    }

    classes = all.vars(formula)

    # make sure that in the data set all column vectors of classes are character vectors
    for (cl in classes) {
      data[,cl]=as.character(data[,cl])
    }
     
    # content lexome is to be specified as the first term in the formula
    content_lexomes = as.character(unique(data[,classes[1]]))
    n_content_lexomes = length(content_lexomes)
    content_lexome_means = rnorm(n_content_lexomes, mean_for_content_lexomes, stdev_for_content_lexomes)
    names(content_lexome_means) = content_lexomes

    # extract unique inflectional features
    features = list()
    for (i in 2:length(classes)) {
      v = data[,classes[i]]
      v = v[!is.na(v)]
      features[[i-1]] = unique(v)
    }
    inflectional_features = unlist(features)
    n_inflectional_features = length(inflectional_features)

    inflectional_features_means = rnorm(n_inflectional_features, 
                                        mean_for_inflectional_features, 
                                        stdev_for_inflectional_features)
    names(inflectional_features_means) = inflectional_features

    M = matrix(rnorm(ncols*nrows,0,1), ncols, nrows)

    if (with_mvrnorm) {
      # for each inflected form, we sum the means of its content lexomes and inflectional features
      n_wordforms = nrows
      mu = rep(0, n_wordforms)
      for (i in 1:n_wordforms) {
        v = as.vector(data[i, classes])
        mu[i]=0
        if (!is.na(v[1][1,1])) mu[i] = mu[i]+unname(content_lexome_means[v[1][1,1]])
        for (j in 2:length(v)) {
          if (!is.na(v[j][1,1])) mu[i] = mu[i] + unname(inflectional_features_means[v[j][1,1]])
        }
      }
  
      # variance-covariance matrix
      Sigma = matrix(0, n_wordforms, n_wordforms)
      rownames(Sigma) = colnames(Sigma) = data[, wordform]
      for (i in 1:n_wordforms) {
        vi = as.vector(data[i,classes])
         
        #for (j in 1:n_wordforms) {
        # fill lower triangle
        for (j in 1:i) {
          vj = as.vector(data[j,classes])
          identical = data[i,classes]==data[j,classes]
          identical[is.na(identical)]=FALSE
  
          vals = data[i,classes][identical]
          # if wordforms share a class feature, the covariance should be updated with
          # covar_for_content_lexomes for content lexomes and with covar_for_inflectional_features otherwise
           
          # content lexomes
          if (identical[1]) {
              Sigma[i,j]=Sigma[i,j]+covar_for_content_lexomes
          }
  
          # other inflectional classes
          for (k in 2:length(identical)) {
            if (identical[k]) {
              Sigma[i,j]=Sigma[i,j]+covar_for_inflectional_features
            }
          }
        }
      }
  
      SigmaT = t(Sigma)
      SigmaT[lower.tri(SigmaT)] = Sigma[lower.tri(Sigma)]
      Sigma = SigmaT
      rm(SigmaT)

      S = MASS::mvrnorm(ncols, mu, Sigma)   # typically, set row dimension to number of cues
      colnames(S) = data[,wordform]
      rownames(S) = paste(rep("S", nrow(S)), 1:nrow(S), sep = "")

      S = S + M
    } 

    if (with_semantic_vectors) {
      Q = matrix(0, ncols, n_inflectional_features+n_content_lexomes)
      colnames(Q) = c(inflectional_features, content_lexomes)
      rownames(Q) = paste(rep("S", nrow(Q)), 1:nrow(Q), sep = "")



      vector_of_means = c(inflectional_features_means, content_lexome_means)
      vector_of_stdevs = c(rep(stdev_for_inflectional_semantic_vectors, n_inflectional_features),
                           rep(stdev_for_lexomic_semantic_vectors, n_content_lexomes))
      for (i in 1:ncol(Q)) {
        Q[,i] = rnorm(nrow(Q), vector_of_means[colnames(Q)[i]], vector_of_stdevs[i])
      }


      if (with_mvrnorm) {
        S1 = S
      } else {
        S1 = M
      }

      for (i in 1:ncol(S1)) {
        # for any given word form, the vectors of all its inflectional vectors are summed
        v = data[i,classes]
        #base = v[1]
        #v = v[-1]
        v = v[!is.na(v)]
        for (j in 1:length(v)) {
          S1[,i] = S1[,i]+Q[,v[j]]
        }
      }
      colnames(S1) = data[,wordform]
      rownames(S1) = paste(rep("S", nrow(S1)), 1:nrow(S1), sep = "")
      S = S1
    } 

    return(t(S))
}
