speak <-
  function(pos = 1,              # row number in dataset
           grams=3,              # trigrams (3) or quadrograms (4)
           threshold=0.1,        # threshold for thinning graph
           amat=am,              # adjacency matrix full dataset
           data=latin,           # dataset
           C = C,                # form matrix with words on rows
           S = S,                # semantic matrix with words on rows
           Bcomp = B,            # beta matrix mapping form onto meaning
           Bprod = NULL,         # used only for analysis by synthesis, together with s
           Chat = Chat,          # predicted form support, words on rows
           s = NULL,             # if not null, s should be semantic vector
                                 # useful when calling speak for analysis by synthesis,
                                 # in this case, pos is effectively not used;
                                 # Bprod should be specified
           include_long_cycles = FALSE,  # if TRUE, cycles longer than 3 are searched for
           m = NULL,             # if include_long_cycles = T, m should be set to output of learn_mappings
           tolerance = 0,        # synthesis_by_analysis tolerance
           returnlist=FALSE,     # return list of paths and their weights
           returnnet=FALSE,      # return thinned adjacency matrix 
           returngraph=FALSE) {  # return thinned graph
    

    if (include_long_cycles & is.null(m)) {
      stop("please specify m when include_long_cycles is set to TRUE\n")
    }

    #----------------------------------------------------------------------
    # extract target word from dataset
    #----------------------------------------------------------------------
    word = data$Word[pos]
    rowpos = pos
    
    #----------------------------------------------------------------------
    # thin the network and make graph
    #----------------------------------------------------------------------
    
    if (!is.null(s)) {
      analysis_by_synthesis=TRUE
      w = (s %*% Bprod)[1,]
    } else {
      w = Chat[pos,]
      analysis_by_synthesis=FALSE
    }
    
    #----------------------------------------------------------------------
    # add support to vertices, and set all supports below threshold to zero
    #----------------------------------------------------------------------

    net = amat
    for (row in 1:length(w)) {
      for (col in 1:length(w)) {
        if (net[row,col]==1) {
          val =  w[colnames(net)[col]]
          if (val > threshold) {
            net[row,col] = w[colnames(net)[col]]
          } else {
            net[row,col] = 0
          }
        }
      }
    }
    #----------------------------------------------------------------------
    # remove vertices without edges from the resulting matrix
    #----------------------------------------------------------------------
    net = round(net, 2)
    row_zeroes = apply(net, 1, FUN=function(v)sum(v==0))
    col_zeroes = apply(net, 2, FUN=function(v)sum(v==0))
    p = which(row_zeroes == ncol(net) & col_zeroes == nrow(net))
    net = net[-p, -p]
    #----------------------------------------------------------------------
    # build graph from adjacency matrix
    #----------------------------------------------------------------------
    net.g = igraph::graph.adjacency(net, mode="directed", weighted=TRUE)
    
    #----------------------------------------------------------------------
    # extract all simple paths from the graph that start at an initial
    # triphone and end at a final triphone
    # ---------------------------------------------------------------------
    
    vertices = colnames(net)
    beg = vertices[grep("^#", vertices)]  
    end = vertices[grep("#$", vertices)]  
    li = list()
    cnt = 0
    for (i_beg in 1:length(beg)) {
      paths = igraph::all_simple_paths(net.g, from=beg[i_beg], to = end)
      if (length(paths) > 0) {
        for (i in 1:length(paths)) {
          cnt = cnt + 1
          li[[cnt]] = get_path_weights(i, net, paths)
        }
      }
    }
    
    
    #------------------------------------------------------------------
    # find cases with triangle cycles and add extended weighted paths
    # to list li 
    #------------------------------------------------------------------
    
    #----------------------------------------------------------------------
    # detect triangles, if any, using igraph function triangles()
    #----------------------------------------------------------------------
    tris = matrix(igraph::triangles(net.g), nrow=3)  
    newpaths=list()  # defines list for added paths
    newpaths_cnt=0   # counter for number of added paths
    #----------------------------------------------------------------------
    # if there are triangles
    #----------------------------------------------------------------------
    if (ncol(tris) > 0) {   
      # define legal orders for cycle with three edges
      cycle_orders = list()
      cycle_orders[[1]]=c(1, 2, 3)
      cycle_orders[[2]]=c(2, 3, 1)
      cycle_orders[[3]]=c(3, 1, 2)
      
      # loop over the cycles detected 
      for (cycle_num in 1:ncol(tris)) {
        # create character vector with the vertices of the cycle
        cycle =  colnames(net)[as.vector(tris[,cycle_num])]
        # make sure the vertices are in a legitimate order for a cycle
        if(substr(cycle[1], 2, 3) != substr(cycle[2], 1, 2)) {
          cycle = cycle[c(2,1,3)]
        }
        
        # loop over paths in list li and create new path
        # with the cycle inserted, if this is  possible
        for (i in 1:length(li)) {
          current_path = names(li[[i]])
          
          # check that cycle is not already used
          if (sum(is.element(current_path, cycle)) > 0) {
            
            # check whether the cycle is already instantiated in the current path
            cycleExists=FALSE
            p = which(current_path %in% cycle)[1]
            for (jj in 1:3) {
              cy = cycle[cycle_orders[[jj]]]
              if ((length(current_path)-p) > 2) {
                if (sum(current_path[p:(p+2)] == cy) == 3) {
                  cycleExists=TRUE
                }
              }
            }
            if (!cycleExists) {
              
              # find first vertex in the current path where a cycle can start
              # --------------------------------------------------------------
              # WARNING: we capitalize on there being only one such vertex
              # --------------------------------------------------------------
              first = current_path[which(current_path %in% cycle)][1]
              if (length(first) > 0) {  # so there is a starting point
                for (j in 1:3) {   # get cycle in correct order
                  cy = cycle[cycle_orders[[j]]]
                  if (cy[1] == first) 
                    break
                }
                cycle = cy
                
                # add cycle into the path
                pos = which(current_path == first)
                newpath = c(current_path[1:(pos - 1)], 
                            cycle, 
                            current_path[pos:length(current_path)])
                
                # add information on support from the mapping
                pw=rep(0, length(newpath))
                for (ii in 1:(length(newpath)-1)) {
                  pw[ii+1] = net[newpath[ii],newpath[ii+1]]
                }
                names(pw)=newpath
                newpath = pw
                
                # increment counter and list of new paths
                newpaths_cnt = newpaths_cnt + 1
                newpaths[[newpaths_cnt]] = newpath
                
              }
            }
          }
        }
      }
      # if new paths have been detected, add these to the list of paths li
      if (length(newpaths) > 0) {
        li = c(li, newpaths)
      }
    }
    
    #------------------------------------------------------------------
    # find cases with two-step cycles and add extended weighted paths
    # with such cycles to list li 
    #------------------------------------------------------------------
    
    #----------------------------------------------------------------------
    # now check for mutual edges using which_mutual from igraph
    #----------------------------------------------------------------------
    mutuals = attr(igraph::E(net.g)[igraph::which_mutual(net.g)], "vnames")
    n_edges=0
    newpaths=list()
    newpaths_cnt=0
    if (length(mutuals) > 0) {
      edges = strsplit(mutuals, "\\|")
      n_edges = length(edges)
      # WARNING: 
      # the code currently assumes that there is only one instance of a mutual edge
      edges = edges[[1]]   # !!
      
      # loop over the paths and check whether the 2-step cycle can be added in
      for (i in 1:length(li)) {
        
        current_path = names(li[[i]])
        
        # insert the cycle with the mutual edge at earliest possible point
        # WARNING: there could be later points at which this could also be done,
        # we currently simply take the first that is found
        pos = which(current_path %in% edges)
        if (length(pos) > 0) {
          first_edge = current_path[pos[1]]
          second_edge = edges[edges!=first_edge]
          
          newpath = c(current_path[1:(pos[1]-1)], 
                      first_edge,
                      second_edge, 
                      current_path[pos[1]:length(current_path)])
          
          # add information on support from the mapping
          pw=rep(0, length(newpath))
          for (ii in 1:(length(newpath)-1)) {
            pw[ii+1] = net[newpath[ii],newpath[ii+1]]
          }
          names(pw)=newpath
          newpath = pw
          
          # update count of new paths and add to list of new paths
          newpaths_cnt = newpaths_cnt + 1
          newpaths[[newpaths_cnt]] = newpath
        }
      }

      # if new paths with 2-step cycles have been found, add them to the list li
      if (length(newpaths) > 0) {
        li = c(li, newpaths)
      }
    }

    # if paths with long cycles are requested, further searches for cycles are made
    if (include_long_cycles) {
      lc = long_cycles(i=rowpos, Chat = Chat,
         am = m$am,
         am.g = m$am.g, 
         Cues = m$ms3$datastructures$Cues,     
         grams=grams) 
      if (!is.null(lc$longPaths)) {
        li = c(li, lc$longPaths[[1]])
      }
    }

    
    # create a character vector with the candidate forms from the triphone vectors
    candidates = sapply(li, FUN=function(nv) ngram2word(names(nv), grams=grams))
    
    if (analysis_by_synthesis) {
      return(list(candidates=candidates, li=li, net.g=net.g, net=net))
    }
    
    # ----------------------------------------------------------------------------
    # synthesis by analysis
    # ----------------------------------------------------------------------------
    # each of the candidate forms is checked for faithfulness, i.e.,
    # its cues are input to the form-to-meaning mapping, and the
    # resulting correlation with the semantic vector that is to be realized 
    # in speech is calculated
    # ----------------------------------------------------------------------------
    
    targetSvec = S[word,]         # the semantic vector to be expressed in n-phones
    theCors=rep(0, length(li))     # vector for storing correlations
    for (k in 1:length(li)) {      # loop over list with alternative paths
      cs = names(li[[k]])        # extract cues
      cuevec = rep(0, ncol(C))  # create cue vector
      names(cuevec)=colnames(C)
      cuevec[cs]=1               # set cells for present cues to 1
      svec =  t(as.matrix(as.vector(cuevec))) %*% Bcomp  # predicted semantic vector
      theCors[k] = cor(t(svec), targetSvec)   # correlation with the sem. vec. to be spoken 
    }
    names(theCors) = candidates
    
    #----------------------------------------------------------------------
    # predicted forms are those with the highest correlation; 
    # ties are possible: there can be multiple best correlating forms
    #----------------------------------------------------------------------
    prediction = candidates[which(theCors>=(max(theCors)-tolerance))]
    
    
    #----------------------------------------------------------------------
    # a measure for choosing between vectors with identical maximal correlations,
    # here length in characters of the candidate words would also work
    #----------------------------------------------------------------------
    length_weakest_link_ratio = sapply(li, FUN=function(v)length(v)/min(v[-1]))
    names(length_weakest_link_ratio)=candidates
    
    #----------------------------------------------------------------------
    # if there are multiple paths, select the one with the lowest ratio
    #----------------------------------------------------------------------
    if (length(prediction)>1) {
      prediction = prediction[which(length_weakest_link_ratio[prediction]==
                                      min(length_weakest_link_ratio[prediction]))]
    }
    
    #----------------------------------------------------------------------
    # set up a list to be returned as output
    #----------------------------------------------------------------------

    res = list(word=word,                   # the target word
               prediction=prediction,       # the predicted words
               candidates=candidates,       # the candidate words
               pos=pos,                     # the index of the target word in the dataset
               length_weakest_link_ratio=length_weakest_link_ratio, # the ratios
               cors=theCors,                # the correlations
               threshold=threshold,         # first parameter value
               tolerance=tolerance          # second parameter value
    )
    
    if (returnlist) res[["li"]] = li        # the list of paths and vertex weights
    if (returnnet) res[["net"]] = net       # the adjacency matrix of the thinned graph
    if (returngraph) res[["graph"]] = net.g # the thinned graph object
    
    return(res)
  }

