\name{semantic_space}
\alias{semantic_space}
\title{
Simulate a semantic vector space.
}
\description{
This function constructs a (words by features) matrix \code{S}.  For each word form, a semantic vector is constructed.  When \code{with_mvrnorm} is TRUE, semantic vectors are generated with mvrnorm.  When \code{with_semantic_vectors} is TRUE, semantic vectors are constructed for each content lexome and each inflectional feature, and added for each word form.  The two methods for constructing the semantic matrix can be combined, or used separately.
}
\usage{
semantic_space(formula=~Lexeme+Person+Number+Tense+Voice+Mood,
               wordform="Word", 
               data=latin,
               with_semantic_vectors=TRUE, 
               with_mvrnorm=FALSE, 
               mean_for_content_lexomes=0, 
               mean_for_inflectional_features=0, 
               stdev_for_content_lexomes=1,
               stdev_for_inflectional_features=1,
               covar_for_content_lexomes=40,
               covar_for_inflectional_features=10,
               stdev_for_lexomic_semantic_vectors=4,
               stdev_for_inflectional_semantic_vectors=4,
               nrows=672,ncols=256)
}
\arguments{
\item{formula}{
A formula describing which classes (columns of the input data) should be used to construct semantic vectors; the first class mentioned should be the content lexome of the inflected word
}
\item{wordform}{
The name of the column in \code{data} that specifies word forms
}
\item{data}{
The dataset for which a semantic vector space is to be simulated
}
\item{with_semantic_vectors}{
Logical, if TRUE, semantic vectors will be defined for each content lexome and each inflectional feature, and added to the \code{S} matrix
}
\item{with_mvrnorm}{
Logical, if TRUE, mvrnorm (MASS) will be used to construct correlated semantic vectors
}
\item{mean_for_content_lexomes}{
Contribution from the content lexome to the mean used by rnorm or mvrnorm when generating a semantic vector for a word form
}
\item{mean_for_inflectional_features}{
Contribution from an inflectional feature to the mean used by rnorm or mvrnorm when generating a semantic vector for a word form
}
\item{stdev_for_content_lexomes}{
Contribution from a content lexome to the stdev used by rnorm or mvrnorm when generating a semantic vector for a word form
}
\item{stdev_for_inflectional_features}{
Contribution from an inflectional feature to the stdev used by rnorm or mvrnorm when generating a semantic vector for a word form
}
\item{covar_for_content_lexomes}{
Contribution from a content lexome to the covariance used by mvrnorm when generating a semantic vector for a word form
}
\item{covar_for_inflectional_features}{
Contribution from an inflectional feature to the covariance used by mvrnorm when generating a semantic vector for a word form
}
\item{stdev_for_lexomic_semantic_vectors}{
Contribution to the standard deviation for rnorm when used to generate a semantic vector for a content lexome
}
\item{stdev_for_inflectional_semantic_vectors}{
  Standard deviation for \code{rnorm} used to generate a semantic vector for an inflectional feature.
}
\item{nrows}{
The dimensionality of the row vectors of \code{S}, a good heuristic is to set this to the number of cues \code{nCues}
}
\item{ncols}{
The number of column vectors, this will typically be the same as the number of column vectors of \code{C}, i.e., the number of inflected forms.
}
}
%\details{
%}
\value{
A  matrix of dimension number of word forms by number of semantic features (set to be equal to the number of columns of \code{C}).
}
%\references{
%% ~put references to the literature/web site here ~
%}
\author{
R. H. Baayen and Yu-Ying Chuang
}
%\note{
%%  ~~further notes~~
%}
%% ~Make other sections like Warning with \section{Warning }{....} ~
%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
data(latin)

# produce C and T matrices, lists of cues, outcomes and unique outcomes
ms3 = 
  make_C_and_T_matrices(formula=~ Lexeme + Person + Number + Tense + Voice + Mood,
                        data=latin, 
                        grams=3)
C = ms3$matrices$C

# create semantic space
set.seed(314)
S = semantic_space(~ Lexeme + Person + Number + Tense + Voice + Mood, nrows=nrow(C), ncols=ncol(C), with_semantic_vectors = TRUE, with_mvrnorm = FALSE)

# produce heatmap on output device
\dontrun{
heatmap(cor(t(S)))
}
}
\keyword{misc}
